@@ -57,6 +57,7 @@ module Pipes (
 
     -- * Re-exports
     -- $reexports
+    , module Control.Monad
     , module Control.Monad.IO.Class
     , module Control.Monad.Trans.Class
     , module Control.Monad.Morph
@@ -64,6 +65,7 @@ module Pipes (
     ) where
 
 import Control.Applicative (Applicative(pure, (<*>)), Alternative(empty, (<|>)))
+import Control.Monad (void)
 import Control.Monad.Error (MonadError(..))
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad (MonadPlus(mzero, mplus))
@@ -413,12 +415,21 @@ instance (MonadError e m) => MonadError e (ListT m) where
     catchError l k = Select (catchError (enumerate l) (\e -> enumerate (k e)))
 
 -- | Run a self-contained `ListT` computation
-runListT :: Monad m => ListT m X -> m ()
-runListT l = runEffect (enumerate l)
+runListT :: Monad m => ListT m a -> m ()
+runListT l = runEffect (enumerate (l >> mzero))
 {-# INLINABLE runListT #-}
 
 {-| 'Enumerable' generalizes 'Data.Foldable.Foldable', converting effectful
     containers to 'ListT's.
+
+    Instances of 'Enumerable' must satisfy these two laws:
+
+> toListT (return r) = return r
+>
+> toListT $ do x <- m  =  do x <- toListT m
+>              f x           toListT (f x)
+
+    In other words, 'toListT' is monad morphism.
 -}
 class Enumerable t where
     toListT :: Monad m => t m a -> ListT m a
@@ -494,6 +505,8 @@ p2 <-< p1 = p1 >-> p2
 {-# INLINABLE (<-<) #-}
 
 {- $reexports
+    "Control.Monad" re-exports 'void'
+
     "Control.Monad.IO.Class" re-exports 'MonadIO'.
 
     "Control.Monad.Trans.Class" re-exports 'MonadTrans'.
